@@ -13,9 +13,10 @@
 
 package PDL::Graphics::Simple::PGPLOT;
 
+use strict;
+use warnings;
 use File::Temp qw/tempfile/;
 use PDL::Options q/iparse/;
-use IPC::Open2;
 
 use PDL;
 
@@ -24,58 +25,60 @@ our $mod = {
     module=>'PDL::Graphics::Simple::PGPLOT',
     engine => 'PDL::Graphics::PGPLOT::Window',
     synopsis=> 'PGPLOT (venerable but trusted)',
-    pgs_version=> '1.010',
+    pgs_api_version=> '1.011',
 };
-eval { require PDL::Graphics::PGPLOT::Window; 1; } and
-  PDL::Graphics::Simple::register( 'PDL::Graphics::Simple::PGPLOT' );
+PDL::Graphics::Simple::register( $mod );
 print $@;
 
-##########
-# PDL::Graphics::Simple::PGPLOT::check
-# Checker
-
 sub check {
-    my $force = shift;
-    $force = 0 unless(defined($force));
-    return $mod->{ok} unless( $force or !defined($mod->{ok}) );
-    eval 'use PDL::Graphics::PGPLOT::Window;';
-    if ($@) {
-	$mod->{ok} = 0;
-	$mod->{msg} = $@;
-	return 0;
+  my $force = shift;
+  $force = 0 unless(defined($force));
+  return $mod->{ok} unless( $force or !defined($mod->{ok}) );
+  eval { require PDL::Graphics::PGPLOT::Window; PDL::Graphics::PGPLOT::Window->import; };
+  if ($@) {
+    $mod->{ok} = 0;
+    $mod->{msg} = $@;
+    return 0;
+  }
+  # Module loaded OK, now try to extract valid devices from it
+  eval {
+    my %devs;
+    PGPLOT::pgqndt(my $n);
+    for my $count (1..$n) {
+      PGPLOT::pgqdt($count,my ($type,$v1,$descr,$v2,$v3));
+      $devs{substr $type, 1} = 1; # chop off "/"
     }
-    # Module loaded OK, now try to extract valid devices from it
-    my @lines = eval {
-      open2(my $chld_out, my $chld_in, $^X, qw(-MPGPLOT -e pgopen(q[?])));
-      print $chld_in "?\n";
-      close $chld_in;
-      grep /^\s+\//, <$chld_out>;
-    };
-    if ($@) {
-	$mod->{ok} = 0;
-	$mod->{msg} = $@;
-	return 0;
-    }
-    $mod->{devices} = { map { chomp; s/^\s*\///; s/\s.*//; ($_,1) } @lines };
-    delete $mod->{disp_dev};
+    $mod->{devices} = \%devs;
+  };
+  if ($@) {
+    $mod->{ok} = 0;
+    $mod->{msg} = $@;
+    return 0;
+  }
+  delete $mod->{disp_dev};
+  if ($ENV{PDL_SIMPLE_DEVICE} || $ENV{PGPLOT_DEV}) {
+    $mod->{disp_dev} = $ENV{PDL_SIMPLE_DEVICE} || $ENV{PGPLOT_DEV};
+    $mod->{disp_dev} =~ s#^/+##;
+  } else {
     TRY:for my $try(qw/XWINDOW XSERVE CGW GW/){
-	if($mod->{devices}->{$try}) { 
-	    $mod->{disp_dev} = $try;
-	    last TRY;
-	}
+      if($mod->{devices}->{$try}) { 
+        $mod->{disp_dev} = $try;
+        last TRY;
+      }
     }
-    unless(exists($mod->{disp_dev})){
-	$mod->{ok} = 0;
-	$mod->{msg} = "Couldn't identify a PGPLOT display device -- giving up.\n";
-	return 0;
-    }
-    unless( $mod->{devices}->{'VCPS'} ) {
-	$mod->{ok} = 0;
-	$mod->{msg} = "Couldn't find the VCPS file-output device -- giving up.\n";
-	return 0;
-    }
-    $mod->{ok} = 1;
-    return 1;
+  }
+  unless(exists($mod->{disp_dev})){
+    $mod->{ok} = 0;
+    $mod->{msg} = "Couldn't identify a PGPLOT display device -- giving up.\n";
+    return 0;
+  }
+  unless( $mod->{devices}{VCPS} ) {
+    $mod->{ok} = 0;
+    $mod->{msg} = "Couldn't find the VCPS file-output device -- giving up.\n";
+    return 0;
+  }
+  $mod->{ok} = 1;
+  return 1;
 }
 
 ##########
@@ -96,13 +99,11 @@ sub new {
     my $pkg = shift;
     my $opt_in = shift;
     my $opt = { iparse( $new_defaults, $opt_in ) };
-    
-    my $pgw;
-    
+
     # Force a recheck on failure, in case the user fixed PGPLOT.
     # Also loads PDL::Graphics::PGPLOT::Window.
     unless(check()) {
-	die "$mod->{shortname} appears nonfunctional\n" unless(check(1));
+	die "$mod->{shortname} appears nonfunctional: $mod->{msg}\n" unless(check(1));
     }
 
     # Figure the device name and size to feed to PGPLOT.
@@ -111,7 +112,7 @@ sub new {
     my $dev;
 
     if( $opt->{type} =~ m/^i/i) {
-	$dev = ( defined($opt->{output}) ? $opt->{output} : "" ) . ($ENV{PGPLOT_DEV} || "/$mod->{disp_dev}");
+	$dev = ( $opt->{output} // "" ) . "/$mod->{disp_dev}";
     } else {
 	my $ext;
 
@@ -126,8 +127,7 @@ sub new {
 	    $opt->{output} .= ".png";
 	}
 
-	our $mod;
-	unless(  $filetypes->{$ext}  and  $mod->{devices}->{$filetypes->{$ext}} ) {
+	unless ($filetypes->{$ext}  and  $mod->{devices}{$filetypes->{$ext}}) {
 	    my($fh);
 	    ($fh, $conv_tempfile) = tempfile('pgs_pgplot_XXXX');
 	    close $fh;
@@ -139,21 +139,15 @@ sub new {
 	}
     }
 
-    ($ENV{'PGPLOT_PS_WIDTH'}) = $opt->{size}->[0] * 1000;
-    ($ENV{'PGPLOT_PS_HEIGHT'}) = $opt->{size}->[1] * 1000;
+    $ENV{PGPLOT_PS_WIDTH} = $opt->{size}[0] * 1000;
+    $ENV{PGPLOT_PS_HEIGHT} = $opt->{size}[1] * 1000;
 
-    my @params = ( 'size => [$opt->{size}->[0], $opt->{size}->[1] ]' );
-    if( defined($opt->{multi}) ) {
-	push(@params, 'nx=>$opt->{multi}->[0]');
-	push(@params, 'ny=>$opt->{multi}->[1]');
-    }
-    
-    my $creator = 'pgwin( $dev, { '. join(",", @params) . '} );';
-    $pgw = eval $creator;
-    print STDERR $@ if($@);
-    
+    my @params = (size => [@{$opt->{size}}[0,1]]);
+    push @params, nx=>$opt->{multi}[0], ny=>$opt->{multi}[1]
+      if defined $opt->{multi};
+    my $pgw = pgwin( $dev, { @params } );
     my $me = { opt=>$opt, conv_fn=>$conv_tempfile, obj=>$pgw };
-    return bless($me, 'PDL::Graphics::Simple::PGPLOT');
+    return bless $me;
 }
 
 our $pgplot_methods = {
@@ -180,10 +174,10 @@ our $pgplot_methods = {
     },
     'labels'=> sub {
 	my ($me,$ipo,$data,$ppo) = @_;
-	for my $i(0..$data->[0]->dim(0)-1) {
-	    $s = $data->[2]->[$i];
+	for my $i (0..$data->[0]->dim(0)-1) {
+	    my $s = $data->[2]->[$i];
 	    my $j = 0.0;
-	    if( $s =~ s/^([\<\|\>\ ])// ) {
+	    if ( $s =~ s/^([\<\|\>\ ])// ) {
 		$j = 0.5 if($1 eq '|');
 		$j = 1.0 if($1 eq '>');
 	    }
@@ -193,9 +187,6 @@ our $pgplot_methods = {
 	
 };
 
-##############################
-# PDL::Graphics::Simple::PGPLOT::plot
-
 sub plot {
     my $me = shift;
     my $ipo = shift;
@@ -204,13 +195,13 @@ sub plot {
     $po->{xtitle}=  $ipo->{xtitle}   if(defined($ipo->{xtitle}));
     $po->{ytitle}=  $ipo->{ytitle}   if(defined($ipo->{ytitle}));
     $po->{justify}= $ipo->{justify}  if(defined($ipo->{justify})); 
-   
-    my %color_opts = ();
-    if(defined($ipo->{crange})) {
-	$color_opts{'MIN'} = $ipo->{crange}->[0] if(defined($ipo->{crange}->[0]));
-	$color_opts{'MAX'} = $ipo->{crange}->[0] if(defined($ipo->{crange}->[1]));
+
+    my %color_opts;
+    if (defined($ipo->{crange})) {
+	$color_opts{MIN} = $ipo->{crange}[0] if defined $ipo->{crange}[0];
+	$color_opts{MAX} = $ipo->{crange}[1] if defined $ipo->{crange}[1];
     }
-    
+
     my $more = 0;
 
     if($ipo->{oplot}  and   $me->{opt}->{type} =~ m/^f/i) {
@@ -218,28 +209,21 @@ sub plot {
     }
 
     unless($ipo->{oplot}) {
-
 	$me->{curvestyle} = 0;
-
 	$me->{logaxis} = $ipo->{logaxis};
-
 	$po->{axis} = 0;
 	if($ipo->{logaxis} =~ m/x/i) {
 	    $po->{axis} += 10;
-	    $ipo->{xrange} = [ log10($ipo->{xrange}->[0]), log10($ipo->{xrange}->[1]) ];
+	    $ipo->{xrange} = [ map log10($_), @{$ipo->{xrange}}[0,1] ];
 	}
 	if($ipo->{logaxis} =~ m/y/i) {
 	    $po->{axis} += 20;
-	    $ipo->{yrange} = [ log10($ipo->{yrange}->[0]), log10($ipo->{yrange}->[1]) ];
+	    $ipo->{yrange} = [ map log10($_), @{$ipo->{yrange}}[0,1] ];
 	}
-	
 	$me->{obj}->release;
 	$me->{obj}->env(@{$ipo->{xrange}}, @{$ipo->{yrange}}, $po);
-	$me->{obj}->hold;
-    } else {
-	# Force a hold if we are held.
-	$me->{obj}->hold;
     }
+    $me->{obj}->hold;
 
     # ppo is "post-plot options", which are really a mix of plot and curve options.  
     # Currently we don't parse any plot options into it (they're handled by the "env"
@@ -247,56 +231,41 @@ sub plot {
     # are curve options that are autoincremented each curve.
     my %ppo = ();
     my $ppo = \%ppo;
-    
-    while(@_) {
+    while (@_) {
 	my ($co, @data) = @{shift()};
 	my @extra_opts = ();
-
-	if( defined($co->{style}) and $co->{style} ) {
+	if ( defined($co->{style}) and $co->{style} ) {
 	    $me->{curvestyle} = int($co->{style});
 	} else {
 	    $me->{curvestyle}++;
 	}
-
-	$ppo->{ color } = ($me->{curvestyle}-1) % 7 + 1;
+	$ppo->{ color } = $me->{curvestyle}-1 % 7 + 1;
 	$ppo->{ linestyle } = ($me->{curvestyle}-1) % 5 + 1;
-
-
-	if( defined($co->{width}) and $co->{width} ) {
-	    $ppo->{ linewidth } = int($co->{width});
-	}
-
+	$ppo->{ linewidth } = int($co->{width}) if $co->{width};
 	our $pgplot_methods;
 	my $pgpm = $pgplot_methods->{$co->{with}};
 	die "Unknown curve option 'with $co->{with}'!" unless($pgpm);
-
 	if($pgpm eq 'imag') {
 	    for my $k(keys %color_opts) {
-		$po->{$k} = $color_opts{$k};
+		$ppo->{$k} = $color_opts{$k};
 	    }
-
 	    $ppo->{ drawwedge } = ($ipo->{wedge} != 0);
-
 	    # Extract transform parameters from the corners of the image...
 	    my $xcoords = shift(@data);
 	    my $ycoords = shift(@data);
-
 	    my $datum_pix = [0,0];
 	    my $datum_sci = [$xcoords->at(0,0), $ycoords->at(0,0)];
-	    
 	    my $t1 = ($xcoords->slice("(-1),(0)") - $xcoords->slice("(0),(0)")) / ($xcoords->dim(0)-1);
 	    my $t2 = ($xcoords->slice("(0),(-1)") - $xcoords->slice("(0),(0)")) / ($xcoords->dim(1)-1);
 	    
 	    my $t4 = ($ycoords->slice("(-1),(0)") - $ycoords->slice("(0),(0)")) / ($ycoords->dim(0)-1);
 	    my $t5 = ($ycoords->slice("(0),(-1)") - $ycoords->slice("(0),(0)")) / ($ycoords->dim(1)-1);
-	    
 	    my $transform = pdl(
 		$datum_sci->[0] - $t1 * $datum_pix->[0] - $t2 * $datum_pix->[1],
 		$t1, $t2,
 		$datum_sci->[1] - $t4 * $datum_pix->[0] - $t5 * $datum_pix->[1],
 		$t4, $t5
 		)->flat;
-
 	    {   # sepia color table
 		my $r = (xvals(256)/255)->sqrt;
 		my $g = (xvals(256)/255);
@@ -304,33 +273,24 @@ sub plot {
 		$me->{obj}->ctab($g, $r, $g, $b);
 	    }
 	}
-
-	if($me->{logaxis} =~ m/x/i) {
-	    $data[0] = $data[0]->log10;
-	}
-	if($me->{logaxis} =~ m/y/i) {
-	    $data[1] = $data[1]->log10;
-	}
-
-	if(ref($pgpm) eq 'CODE') {
-	    &$pgpm($me, $ipo, \@data, $ppo);
+	$data[0] = $data[0]->log10 if $me->{logaxis} =~ m/x/i;
+	$data[1] = $data[1]->log10 if $me->{logaxis} =~ m/y/i;
+	if (ref $pgpm eq 'CODE') {
+	    $pgpm->($me, $ipo, \@data, $ppo);
 	} else {
-	    my $str= sprintf('$me->{obj}->%s(@data,%s);%s',$pgpm,'$ppo',"\n");
-	    eval $str;
+	    $me->{obj}->$pgpm(@data,$ppo);
 	}
-
 	unless($pgpm eq 'imag') {
 	    $ppo->{linestyle}++;
 	    $ppo->{color}++;
 	}
-
 	$me->{obj}->hold;
     }
 
     ##############################
     # End of curve plotting.
     # Now place the legend if necessary.
-    if($ipo->{legend}) {
+    if ($ipo->{legend}) {
 	my $xp;
 	my $xrdiff = $ipo->{xrange}->[1] - $ipo->{xrange}->[0];
 	if( $ipo->{legend}=~ m/l/i ) {
@@ -340,7 +300,6 @@ sub plot {
 	} else {
 	    $xp = 0.4 * $xrdiff + $ipo->{xrange}->[0];
 	}
-
 	my $yp;
 	my $yrdiff = $ipo->{yrange}->[1] - $ipo->{yrange}->[0];
 	if( $ipo->{legend}=~ m/t/i ) {
@@ -350,7 +309,6 @@ sub plot {
 	} else {
 	    $yp = 0.6 * $yrdiff + $ipo->{yrange}->[0];
 	}
-
 	print "keys is [".join(",",@{$me->{keys}})."]; xp is $xp; yp is $yp\n";
 	$me->{obj}->legend( 
 	    $me->{keys},
@@ -360,26 +318,23 @@ sub plot {
 	    }
 	    );
     }
-
     $me->{obj}->release;
-
 }
 
-
 sub DESTROY {
     my $me = shift;
 
-    $me->{obj}->release;
+    eval { # in case of global destruction
+      $me->{obj}->release;
+    };
 
-    if($me->{type} =~ m/^f/i) {
-	eval q{ $me->{obj}->close; };
-
-	my $file = ( ($me->{conv_fn}) ? $me->{conv_fn} : $me->{output} );
-	if($me->{conv_fn}) {
-	    my $a;
-	    $a = rim($me->{conv_fn}) ;
-	    wim($a, $me->{opt}->{output}); 
+    if (defined $me->{type} and $me->{type} =~ m/^f/i) {
+	eval { $me->{obj}->close; };
+	if ($me->{conv_fn}) {
+	    wim(rim($me->{conv_fn}), $me->{opt}{output});
 	    unlink($me->{conv_fn});
 	}
     }
 }
+
+1;
