@@ -7,8 +7,8 @@
 ###
 ### See the PDL::Graphics::Simple docs for details
 ###
-### Prima setup is borrowed from D. Mertens' PDL::Graphics::Prima::Simple 
-###  
+### Prima setup is borrowed from D. Mertens' PDL::Graphics::Prima::Simple
+###
 ##
 #
 
@@ -17,8 +17,9 @@
 
 
 package PDL::Graphics::Simple::Prima;
-use strict;
 
+use strict;
+use warnings;
 use PDL;
 use PDL::Options q/iparse/;
 use File::Temp qw/tempfile/;
@@ -28,10 +29,11 @@ our $mod = {
     module => 'PDL::Graphics::Simple::Prima',
     engine => 'PDL::Graphics::Prima',
     synopsis => 'Prima (interactive, fast, PDL-specific)',
-    pgs_version => '1.010',
+    pgs_api_version=> '1.011',
 };
-eval { require PDL::Graphics::Prima; 1 } and
-  PDL::Graphics::Simple::register('PDL::Graphics::Simple::Prima');
+PDL::Graphics::Simple::register( $mod );
+
+our (@colors, @patterns, $types);
 
 ##########
 # PDL::Graphics::Simple::Prima::check
@@ -39,7 +41,7 @@ eval { require PDL::Graphics::Prima; 1 } and
 sub check {
     my $force = shift;
     $force = 0 unless(defined($force));
-    
+
     return $mod->{ok} unless( $force or !defined($mod->{ok}));
     $mod->{ok} = 0; # makes default case simpler
 
@@ -62,7 +64,7 @@ sub check {
 	undef $@;
 	return 0;
     }
-    
+
     eval {
 	require Prima::Application;
 	Prima::Application->import();
@@ -81,13 +83,24 @@ sub check {
 	require Prima::Buttons;
 	require Prima::Utils;
 	require Prima::Edit;
+	require Prima::Const;
     };
-    if($@){ 
+    if($@){
 	$mod->{msg} = "Couldn't load auxiliary Prima modules: ".$@;
 	undef $@;
 	return 0;
     }
- 
+    @colors = (
+      cl::Black(), cl::Red(), cl::Green(), cl::Blue(), cl::Cyan(),
+      cl::Magenta(), cl::Yellow(), cl::Brown(),
+      cl::LightRed(), cl::LightGreen(), cl::LightBlue(), cl::Gray(),
+    );
+    @patterns = (
+      lp::Solid(), lp::Dash(), lp::LongDash(), lp::ShortDash(), lp::DotDot(),
+      lp::DashDot(), lp::DashDotDot(),
+    );
+    _load_types();
+
     $mod->{ok} =1;
     return 1;
 }
@@ -111,40 +124,37 @@ sub new {
     $opt_in = {} unless(defined($opt_in));
     my $opt = { iparse($new_defaults, $opt_in) };
 
-    
+
     unless( check() ) {
-	die "$mod->{shortname} appears nonfunctional\n" unless(check(1));
+	die "$mod->{shortname} appears nonfunctional: $mod->{msg}\n" unless(check(1));
     }
 
     my $size = PDL::Graphics::Simple::_regularize_size($opt->{size},'px');
 
     my $pw = Prima::Window->create( text => $opt->{output} || "PDL/Prima Plot",
 				    size => [$size->[0], $size->[1]],
-				    onCreate  => sub { $PDL::Graphics::Prima::Simple::N_windows++; },  
-				    onDestroy => sub { $PDL::Graphics::Prima::Simple::N_windows--; 
+				    onCreate  => sub { $PDL::Graphics::Prima::Simple::N_windows++; },
+				    onDestroy => sub { $PDL::Graphics::Prima::Simple::N_windows--;
 						       PDL::Graphics::Prima::Simple::twiddling(0) if($PDL::Graphics::Prima::Simple::N_windows==0);
-				    } 	
-	);                         
+				    }
+	);
     die "Couldn't create a Prima window!" unless(defined($pw));
 
     if($opt_in->{type} =~ m/^f/i) {
 	$pw->hide;
     }
     my $me = { obj => $pw,
-	       widgets => [], 
-	       next_plotno=>0, 
-	       multi=>$opt_in->{multi}, 
-	       type=>$opt->{type}, 
-	       output=>$opt->{output} 
+	       widgets => [],
+	       next_plotno=>0,
+	       multi=>$opt_in->{multi},
+	       type=>$opt->{type},
+	       output=>$opt->{output}
     };
     return bless($me, "PDL::Graphics::Simple::Prima");
 }
 
 sub DESTROY {
     my $me = shift;
-
-    print "DESTROYING $me\n";
-
     if($me->{type} =~ m/f/i) {
 	##############################
 	# File-saving code...
@@ -154,11 +164,11 @@ sub DESTROY {
 	    # Save plot to file
 
 	    if($me->{widgets}->[0]) {
-		eval q{$me->{widgets}->[0]->save_to_file($me->{output})};
+		eval {$me->{widgets}->[0]->save_to_file($me->{output})};
 		if($@) {
 		    print $@;
 		    undef $@;
-		} 
+		}
 	    } else {
 		print STDERR "No plot was sent to $me->{output}\n";
 	    }
@@ -173,11 +183,11 @@ sub DESTROY {
 		print STDERR "No plot was sent to $me->{output}\n";
 	    } else {
 		print STDERR "WARNING - multiplot support is experimental for the Prima engine\n";
-		
+
 		my ($h,$tmpfile) = tempfile('PDL-Graphics-Simple-XXXX');
 		close $h;
 		unlink($tmpfile);
-		
+
 		my $suffix;
 		if($me->{output}=~ s/(\.\w{2,4})$//) {
 		    $suffix = $1;
@@ -185,18 +195,18 @@ sub DESTROY {
 		    $suffix = ".png";
 		}
 		$tmpfile .= $suffix;
-		
+
 		my $widget_dex = 0;
 		my $im = undef;
 		my $ztile = undef;
 	      ROW:for my $row(0..$me->{multi}->[1]-1) {
 		  my $imrow = undef;
 		  for my $col(0..$me->{multi}->[0]-1) {
-		      
+
 		      my $tile;
-		      
+
 		      if($widget_dex < @{$me->{widgets}}) {
-			  eval q{ $me->{widgets}->[$widget_dex++]->save_to_file($tmpfile) };
+			  eval { $me->{widgets}->[$widget_dex++]->save_to_file($tmpfile) };
 			  last ROW if($@);
 			  $tile = rim($tmpfile);
 			  $ztile = zeroes($tile)+255;
@@ -205,14 +215,14 @@ sub DESTROY {
 			  # ztile is always initialized by first run through...
 			  $tile = $ztile;
 		      }
-		      
+
 		      if(!defined($imrow)) {
 			  $imrow = $tile;
 		      } else {
 			  $imrow = $imrow->glue(0,$tile);
 		      }
 		  } # end of row loop
-		  
+
 		  if(!defined($im)) {
 		      $im = $imrow;
 		  } else {
@@ -229,20 +239,12 @@ sub DESTROY {
 	}
     }
 
-
-
-    $me->{obj}->hide;
-    $me->{obj}->destroy;
+    eval { # in case of global destruction
+      $me->{obj}->hide;
+      $me->{obj}->destroy;
+    };
 }
 
-
-our @colors =qw/
-    cl::Black cl::Red cl::Green cl::Blue cl::Cyan cl::Magenta cl::Yellow cl::Brown cl::LighttRed cl::LightGreen cl::LightBlue cl::Gray/;
-
-our @patterns = qw/
-    lp::Solid lp::Dash lp::LongDash lp::ShortDash lp::DotDot lp::DashDot lp::DashDotDot/;
-
-
 ##############################
 # Fake-o apply method makes sepiatone values for input data.
 # We have to mock up an object method to match the style of PDL::Graphics::Prima::Palette,
@@ -251,161 +253,144 @@ our @patterns = qw/
 sub PDL::Graphics::Simple::Prima::Sepia_Palette::apply {
     my $h = shift;
     my $data = shift;
-
-    my $crange = $h->{crange};
-    my($min, $max);
-    if(defined($crange)){
-	($min,$max) = @$crange;
-    }
-    $min = $data->min unless(defined($min));
-    $max = $data->max unless(defined($max));
-
-    my $g = ($min==$max)?$ data->zeroes : (($data->double - $min)/($max-$min))->clip(0,1);
+    my ($min, $max) = @$h{qw(min max)};
+    my $g = ($min==$max)? $data->zeroes : (($data->double - $min)/($max-$min))->clip(0,1);
     my $r = $g->sqrt;
     my $b = $g*$g;
-    
     return (pdl($r,$g,$b)*255.999)->floor->mv(-1,0)->rgb_to_color;
 }
 
- 
+
 ##############################
 # Plot types
 #
-# This probably needs a little more smarts.  
+# This probably needs a little more smarts.
 # Currently each entry is either a ppair::<foo> return or a sub that implements
-# the plot type in terms of others. 
+# the plot type in terms of others.
+sub _load_types {
+  $types = {
+    lines => [ppair::Lines()],
 
-our $types = {
-    lines => q{ppair::Lines},
-
-    points => [ map { 'ppair::'.$_ } qw/Blobs Triangles Squares Crosses Xs Asterisks/ ],
+    points => [ map ppair->can($_)->(), qw/Blobs Triangles Squares Crosses Xs Asterisks/ ],
 
     bins => sub {
-	my ($me, $plot, $block, $cprops) = @_;
-	my $x = $block->[0];
-	my $x1 = $x->range( [[0],[-1]], [$x->dim(0)], 'e' )->average;
-	my $x2 = $x->range( [[1],[0]],  [$x->dim(0)], 'e' )->average;
-	my $newx = pdl($x1, $x2)->mv(-1,0)->clump(2)->sever;
-
-	my $y = $block->[1];
-	my $newy = $y->dummy(0,2)->clump(2)->sever;
-	
-	$plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = 
-	    ds::Pair($newx,$newy,plotType=>eval q{ppair::Lines}, @$cprops);
+      my ($me, $plot, $block, $cprops) = @_;
+      my $x = $block->[0];
+      my $x1 = $x->range( [[0],[-1]], [$x->dim(0)], 'e' )->average;
+      my $x2 = $x->range( [[1],[0]],  [$x->dim(0)], 'e' )->average;
+      my $newx = pdl($x1, $x2)->mv(-1,0)->clump(2)->sever;
+      my $y = $block->[1];
+      my $newy = $y->dummy(0,2)->clump(2)->sever;
+      $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } =
+          ds::Pair($newx,$newy,plotType=>ppair::Lines(), @$cprops);
     },
 
-
-
     image => sub {
-	my($me, $plot, $data, $cprops, $co) = @_;
-
-	my ($xmin, $xmax) = $data->[0]->minmax;
-	my $dx = 0.5 * ($xmax-$xmin) / ($data->[0]->dim(0) - (($data->[0]->dim(0)==1) ? 0 : 1));
-	$xmin -= $dx;
-	$xmax += $dx;
-
-	my ($ymin, $ymax) = $data->[1]->minmax;
-	my $dy = 0.5 * ($ymax-$ymin) / ($data->[0]->dim(1) - (($data->[1]->dim(1)==1) ? 0 : 1));
-	$ymin -= $dy;
-	$ymax += $dy;
-
-	$plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = 
-	  ds::Grid($data->[2], 
-		   x_bounds=>[ $xmin, $xmax ],
-		   y_bounds=>[ $ymin, $ymax ],
-		   plotType=>pgrid::Matrix( palette => bless({crange=>$me->{ipo}->{crange},data=>$data,co=>$co},'PDL::Graphics::Simple::Prima::Sepia_Palette')),
-	  );
-
-	if(!!$co->{wedge}) {
-	    print STDERR "Color wedges are not supported (yet) in Prima\n";
-	}
+      my ($me, $plot, $data, $cprops, $co, $ipo) = @_;
+      my ($xmin, $xmax) = $data->[0]->minmax;
+      my $dx = 0.5 * ($xmax-$xmin) / ($data->[0]->dim(0) - (($data->[0]->dim(0)==1) ? 0 : 1));
+      $xmin -= $dx;
+      $xmax += $dx;
+      my ($ymin, $ymax) = $data->[1]->minmax;
+      my $dy = 0.5 * ($ymax-$ymin) / ($data->[0]->dim(1) - (($data->[1]->dim(1)==1) ? 0 : 1));
+      $ymin -= $dy;
+      $ymax += $dy;
+      my $dataset;
+      my $imdata = $data->[2];
+      my @bounds = (x_bounds=>[ $xmin, $xmax ], y_bounds=>[ $ymin, $ymax ]);
+      if ($imdata->ndims > 2) {
+        $imdata = $imdata->mv(-1,0) if $imdata->dim(0) != 3;
+        $dataset = ds::Image($imdata, @bounds);
+      } else {
+        my $crange = $me->{ipo}{crange};
+        my ($cmin, $cmax) = defined($crange) ? @$crange : ();
+        $cmin //= $imdata->min;
+        $cmax //= $imdata->max;
+        my $palette = PDL::Graphics::Simple::Prima::Sepia_Palette->new(
+          min => $cmin, max => $cmax, data => $imdata,
+        );
+        $dataset = ds::Grid($imdata,
+          @bounds,
+          plotType=>pgrid::Matrix($ipo->{wedge} ? () : (palette => $palette)),
+        );
+        $plot->color_map($palette) if $ipo->{wedge};
+      }
+      $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = $dataset;
     },
 
-
     circles => sub {
-	my($me, $plot, $data, $cprops) = @_;
-	our $cstash;
-	unless(defined($cstash)) {
-	    my $ang = PDL->xvals(362)*3.14159/180;
-	    $cstash = {};
-	    $cstash->{c}   = $ang->cos;
-	    $cstash->{s}   = $ang->sin;
-	    $cstash->{s}->slice("361") .= $cstash->{c}->slice("361") .= PDL->pdl(1.1)->acos; # NaN
-	}
-	my $dr = $data->[2]->flat;
-	my $dx = ($data->[0]->flat->dummy(0,1) + $dr->dummy(0,1)*$cstash->{c})->flat;
-	my $dy = ($data->[1]->flat->dummy(0,1) + $dr->dummy(0,1)*$cstash->{s})->flat;
-	$plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = 
-	    ds::Pair( $dx, $dy, plotType=>eval q{ppair::Lines}, @$cprops);
+      my ($me, $plot, $data, $cprops) = @_;
+      our $cstash;
+      unless(defined($cstash)) {
+        my $ang = PDL->xvals(362)*3.14159/180;
+        $cstash = {c => $ang->cos, s => $ang->sin};
+        $cstash->{s}->slice("361") .= $cstash->{c}->slice("361") .= PDL->pdl(1.1)->acos; # NaN
+      }
+      my $dr = $data->[2]->flat;
+      my $dx = ($data->[0]->flat->dummy(0,1) + $dr->dummy(0,1)*$cstash->{c})->flat;
+      my $dy = ($data->[1]->flat->dummy(0,1) + $dr->dummy(0,1)*$cstash->{s})->flat;
+      $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } =
+        ds::Pair( $dx, $dy, plotType=>ppair::Lines(), @$cprops);
     },
 
-
     labels => sub {
-	my($me,$plot,$block,$cprops,$co) = @_;
+      my ($me,$plot,$block,$cprops,$co,$ipo) = @_;
+      my ($x, $y) = map $_->flat->sever, @$block[0,1];
+      my @labels = @{$block->[2]};
+      my @lrc = ();
+      for my $i(0..$x->dim(0)-1) {
+        my $j =0;
+        if($labels[$i] =~ s/^([\<\|\> ])//) {
+          my $ch = $1;
+          if($ch =~ m/[\|\>]/) {
+            my $tw = $plot->get_text_width($labels[$i]);
+            $tw /= 2 if($ch eq '|');
+            $x->slice("($i)") .=
+              $plot->x->pixels_to_reals(
+                $plot->x->reals_to_pixels( $x->slice("($i)") ) - $tw
+              );
+          }
+        }
+      }
+      $plot->dataSets()->{1+keys(%{$plot->dataSets()})} =
+        ds::Note(
+          map pnote::Text($labels[$_],x=>$x->slice("($_)"),y=>$y->slice("($_)")), 0..$#labels
+        );
+    },
 
-	my $x = $block->[0]->flat->sever;
-	my $y = $block->[1]->flat->sever;
-	my @labels = @{$block->[2]};
+    limitbars => sub {
+      # Strategy: make T-errorbars out of the x/y/height data and generate a Line
+      # plot.  The T-errorbar width is 4x the LineWidth (+/- 2x).
+      my ($me, $plot, $block, $cprops, $co, $ipo) = @_;
+      my $x = $block->[0]->flat;
+      my $y = $block->[1]->flat;
+      my $ylo = $block->[2]->flat;
+      my $yhi = $block->[3]->flat;
+      # Calculate T bar X ranges
+      my $of = ($co->{width}||1) * 2;
+      my $xp = $plot->x->reals_to_pixels($x);
+      my $xlo = $plot->x->pixels_to_reals(  $xp - $of );
+      my $xhi = $plot->x->pixels_to_reals(  $xp + $of );
+      my $nan = PDL->new_from_specification($x->dim(0));  $nan .= asin(pdl(1.1));
+      my $xdraw = pdl($xlo,$xhi,$x,  $x,  $xlo,$xhi,$nan)->mv(1,0)->flat;
+      my $ydraw = pdl($ylo,$ylo,$ylo,$yhi,$yhi,$yhi,$nan)->mv(1,0)->flat;
+      $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } =
+          ds::Pair($xdraw,$ydraw,plotType=>ppair::Lines(), @$cprops);
+      $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } =
+          ds::Pair($x,$y,plotType=>$types->{points}->[ ($me->{curvestyle}-1) %(0+@{$types->{points}}) ], @$cprops);
+    },
 
-	my @lrc = ();
-	for my $i(0..$x->dim(0)-1) {
-	    my $j =0;
-	    if($labels[$i] =~ s/^([\<\|\> ])//) {
-		my $ch = $1;
-		if($ch =~ m/[\|\>]/) {
-		    my $tw = $plot->get_text_width($labels[$i]);
-		    $tw /= 2 if($ch eq '|');
-		    $x->slice("($i)") .= 
-			$plot->x->pixels_to_reals(
-			    $plot->x->reals_to_pixels( $x->slice("($i)") ) - $tw 
-			);
-		}
-	    }
-	}
-
-	$plot->dataSets()->{1+keys(%{$plot->dataSets()})} = 
-	  ds::Note(
-	      map { eval q{pnote::Text($labels[$_],x=>$x->slice("($_)"),y=>$y->slice("($_)"))}; } (0..$#labels)
-	  );
-    }
-};
-
-$types->{limitbars} = sub {
-    # Strategy: make T-errorbars out of the x/y/height data and generate a Line
-    # plot.  The T-errorbar width is 4x the LineWidth (+/- 2x).
-    my($me, $plot, $block, $cprops, $co) = @_;
-    my $x = $block->[0]->flat;
-    my $y = $block->[1]->flat;
-    my $ylo = $block->[2]->flat;
-    my $yhi = $block->[3]->flat;
-    
-    # Calculate T bar X ranges
-    my $of = ($co->{width}||1) * 2;
-    my $xp = $plot->x->reals_to_pixels($x);
-    my $xlo = $plot->x->pixels_to_reals(  $xp - $of );
-    my $xhi = $plot->x->pixels_to_reals(  $xp + $of );
-    my $nan = PDL->new_from_specification($x->dim(0));  $nan .= asin(pdl(1.1));
-    
-    my $xdraw = pdl($xlo,$xhi,$x,  $x,  $xlo,$xhi,$nan)->mv(1,0)->flat; 
-    my $ydraw = pdl($ylo,$ylo,$ylo,$yhi,$yhi,$yhi,$nan)->mv(1,0)->flat;
-    $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = 
-	ds::Pair($xdraw,$ydraw,plotType=>eval q{ppair::Lines}, @$cprops);
-    $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } =
-	ds::Pair($x,$y,plotType=>eval ($types->{points}->[ ($me->{curvestyle}-1) %(0+@{$types->{points}}) ]), @$cprops);
-};
-
-
-$types->{errorbars} = sub {
-    # Strategy: make T-errorbars out of the x/y/height data and generate a Line
-    # plot.  The T-errorbar width is 4x the LineWidth (+/- 2x).
-    my($me, $plot, $block, $cprops, $co) = @_;
-    my $halfwidth = $block->[2]->flat;
-    $block->[2] = $block->[1] - $halfwidth;
-    $block->[3] = $block->[1] + $halfwidth;
-    &{$types->{limitbars}}($me, $plot, $block, $cprops, $co);
-};	  
-
- 
+    errorbars => sub {
+      # Strategy: make T-errorbars out of the x/y/height data and generate a Line
+      # plot.  The T-errorbar width is 4x the LineWidth (+/- 2x).
+      my ($me, $plot, $block, $cprops, $co, $ipo) = @_;
+      my $halfwidth = $block->[2]->flat;
+      $block->[2] = $block->[1] - $halfwidth;
+      $block->[3] = $block->[1] + $halfwidth;
+      $types->{limitbars}->($me, $plot, $block, $cprops, $co, $ipo);
+    },
+  };
+}
 
 
 ##############################
@@ -421,14 +406,14 @@ sub plot {
     if(defined($ipo->{legend})) {
 	printf(STDERR "WARNING: Ignoring 'legend' option (Legends not yet supported by PDL::Graphics::Simple::Prima v%s)",$PDL::Graphics::Simple::VERSION);
     }
-    
+
     my $plot;
-    
+
     if($ipo->{oplot} and defined($me->{last_plot})) {
 	$plot = $me->{last_plot};
     } else {
 	$me->{curvestyle} = 0;
-	
+
 	if( $me->{multi} ) {
 	    # Multiplot - handle logic and plot placement
 
@@ -461,41 +446,40 @@ sub plot {
 
     push(@{$me->{widgets}}, $plot);
     $me->{last_plot} = $plot;
-    
+
     ## Set global plot options: titles, axis labels, and ranges.
     $plot->hide;
     $plot->lock;
     $plot->title(     $ipo->{title}   )  if(defined($ipo->{title}));
     $plot->x->label(  $ipo->{xlabel}  )  if(defined($ipo->{xlabel}));
     $plot->y->label(  $ipo->{ylabel}  )  if(defined($ipo->{ylabel}));
-    
-    $plot->x->scaling(eval q{sc::Log}) if($ipo->{logaxis}=~ m/x/i);
-    $plot->y->scaling(eval q{sc::Log}) if($ipo->{logaxis}=~ m/y/i);
-    
+
+    $plot->x->scaling(sc::Log()) if($ipo->{logaxis}=~ m/x/i);
+    $plot->y->scaling(sc::Log()) if($ipo->{logaxis}=~ m/y/i);
+
     $plot->x->min($ipo->{xrange}->[0]) if(defined($ipo->{xrange}) and defined($ipo->{xrange}->[0]));
     $plot->x->max($ipo->{xrange}->[1]) if(defined($ipo->{xrange}) and defined($ipo->{xrange}->[1]));
     $plot->y->min($ipo->{yrange}->[0]) if(defined($ipo->{yrange}) and defined($ipo->{yrange}->[0]));
     $plot->y->max($ipo->{yrange}->[1]) if(defined($ipo->{yrange}) and defined($ipo->{yrange}->[1]));
-    
+
     ##############################
-    # I couldn't find a way to scale the plot to make the plot area justified, so 
+    # I couldn't find a way to scale the plot to make the plot area justified, so
     # we cheat and adjust the axis values instead.
     # This is a total hack, but at least it produces justified plots.
     if( !!($ipo->{justify}) ) {
 	my ($dmin,$pmin,$dmax,$pmax,$xscale,$yscale);
-	
+
 	($dmin,$dmax) = $plot->x->minmax;
 	$pmin = $plot->x->reals_to_pixels($dmin);
 	$pmax = $plot->x->reals_to_pixels($dmax);
 	$xscale = ($pmax-$pmin)/($dmax-$dmin);
-	
+
 	($dmin,$dmax) = $plot->y->minmax;
 	$pmin = $plot->y->reals_to_pixels($dmin);
 	$pmax = $plot->y->reals_to_pixels($dmax);
 	$yscale = ($pmax-$pmin)/($dmax-$dmin);
-	
+
 	my $ratio = $yscale / $xscale;
-	print "ratio=$ratio\n";
 	if($ratio > 1) {
 	    # More Y pixels per datavalue than X pixels.  Hence we expand the Y range.
 	    my $ycen = ($dmax+$dmin)/2;
@@ -516,37 +500,31 @@ sub plot {
 
 
     ##############################
-    # Rubber meets the road -- loop over data blocks and 
+    # Rubber meets the road -- loop over data blocks and
     # ship out each curve to the appropriate dispatcher in the $types table
     for my $block(@_) {
 	my $co = shift @$block;
-	
+
 	# Parse out curve style (for points type selection)
 	if(defined($co->{style}) and $co->{style}) {
 	    $me->{curvestyle} = $co->{style};
 	} else {
 	    $me->{curvestyle}++;
 	}
-	
+
 	my $cprops = [
-	    color        => eval $colors[   ($me->{curvestyle}-1) % @colors ],
-	    linePattern  => eval $patterns[ ($me->{curvestyle}-1) % @patterns ],
+	    color        => $colors[   ($me->{curvestyle}-1) % @colors ],
+	    linePattern  => $patterns[ ($me->{curvestyle}-1) % @patterns ],
 	    lineWidth    => $co->{width} || 1
 	    ];
-	
+
 	my $type = $types->{$co->{with}};
+	die "$co->{with} is not yet implemented in PDL::Graphics::Simple for Prima.\n"
+	    if !defined $type;
 	if( ref($type) eq 'CODE' ) {
-	    &{$type}($me, $plot, $block, $cprops, $co);
+	    $type->($me, $plot, $block, $cprops, $co, $ipo);
 	} else {
-	    my $pt;
-	    if(ref($type) eq 'ARRAY') {
-		$pt = eval sprintf("%s",$type->[ ($me->{curvestyle}-1) % (0+@{$type}) ] );
-	    } elsif(!defined($type)) {
-		die "$co->{with} is not yet implemented in PDL::Graphics::Simple for Prima.\n";
-	    } else {
-		$pt = eval qq{$type};
-	    }
-	    
+	    my $pt = ref($type) eq 'ARRAY' ? $type->[ ($me->{curvestyle}-1) % (0+@{$type}) ] : eval $type;
 	    $plot->dataSets()->{ 1+keys(%{$plot->dataSets()}) } = ds::Pair(@$block, plotType => $pt, @$cprops);
 	}
     }
@@ -559,7 +537,7 @@ sub plot {
 	$plot->hide;
 	$me->{obj}->hide;
     }
-    
+
     ##############################
     # Another lame kludge.  Run the event loop for 50 milliseconds, to enable a redraw,
     # then exit it.
@@ -567,10 +545,9 @@ sub plot {
 	onTick=>sub{$_[0]->stop; die "done with event loop\n"},
 	timeout=>50
 	)->start;
-    eval { $::application->go };
+    eval { no warnings 'once'; $::application->go };
     die unless $@ =~ /^done with event loop/;
     undef $@;
+}
 
-}    
-    
-    
+1;
